#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------------------------------------------------------
# Author: Lalith Kumar Shiyam Sundar | Sebastian Gutschmayer
# Institution: Medical University of Vienna
# Research Group: Quantitative Imaging and Medical Physics (QIMP) Team
# Date: 04.07.2023
# Version: 0.1.0
#
# Description:
# This module shows predefined display messages for the pumaz.
#
# Usage:
# The functions in this module can be imported and used in other modules within the pumaz to show predefined display
# messages.
#
# ----------------------------------------------------------------------------------------------------------------------

import logging

import pyfiglet
from pumaz import constants
from rich.console import Console

console = Console()

def logo():
    """
    Display the PUMA logo.

    :return: None
    :rtype: None
    :Example:
        >>> logo()
    """
    print(' ')
    logo_color_code = constants.ANSI_VIOLET
    slogan_color_code = constants.ANSI_VIOLET
    result = logo_color_code + pyfiglet.figlet_format("PUMA 1.0", font="speed").rstrip() + "\033[0m"
    text = slogan_color_code + "A part of the ENHANCE community. Join us at https://enhance.pet to build the future " \
                               "of " \
                               "PET imaging together." + "\033[0m"
    print(result)
    print(text)
    print(' ')


def citation():
    """
    Display the manuscript citation for PUMA.

    :return: None
    :rtype: None
    :Example:
        >>> citation()
    """
    console.print(" TBD", style="white")
    console.print(" Copyright 2023, Quantitative Imaging and Medical Physics Team, Medical University of Vienna",
                  style="white")


def expectations():
    """
    Display the expected modalities for PUMA.

    This function is used to check if the user has provided the correct set of modalities for each tracer set.

    :return: None
    :rtype: None
    :Example:
        >>> expectations()
    """
    # display the expected modalities
    console.print(f' Expected anatomical modalities: {constants.ANATOMICAL_MODALITIES} |'
          f' Number of required anatomical modalities: 1 |'
          f' Expected functional modalities: {constants.FUNCTIONAL_MODALITIES} |'
          f' Number of required functional modalities: 1 |'
                  f' Required prefix for non-DICOM files: {constants.MODALITIES_PREFIX}', style='white')
    logging.info(f' Expected anatomical modalities: {constants.ANATOMICAL_MODALITIES} |'
                 f' Number of required anatomical modalities: 1 |'
                 f' Expected functional modalities: {constants.FUNCTIONAL_MODALITIES} |'
                 f' Number of required functional modalities: 1 |'
                 f' Required prefix for non-DICOM files: {constants.MODALITIES_PREFIX}')
    console.print(
        f" Warning: Any subject datasets in a non-DICOM format that lack the required modalities"
        f" (as indicated by the file prefix) will not be included in the analysis.", style="bold magenta")

    warning_message = " Skipping subjects without the required modalities (check file prefix).\n" \
                      " These subjects will be excluded from analysis and their data will not be used."
    logging.warning(warning_message)




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------------------------------------------------------
# Author: Lalith Kumar Shiyam Sundar | Sebastian Gutschmayer
# Institution: Medical University of Vienna
# Research Group: Quantitative Imaging and Medical Physics (QIMP) Team
# Date: 04.07.2023
# Version: 0.1.0
#
# Description:
# This module contains the urls and filenames of the binaries that are required for the pumaz.
#
# Usage:
# The variables in this module can be imported and used in other modules within the pumaz to download the necessary
# binaries for the pumaz.
#
# ----------------------------------------------------------------------------------------------------------------------
import torch
from rich.console import Console

console = Console()

# GREEDY_BINARIES is a dictionary that contains the download information for the binaries required for the pumaz.
# Each key in the dictionary is a string that represents the name of the binary.
# The value associated with each key is another dictionary that contains the following keys:
# - "url": A string that represents the URL where the binary can be downloaded.
# - "filename": A string that represents the name of the file that will be downloaded.
# - "directory": A string that represents the name of the directory where the downloaded file will be stored.

PUMA_BINARIES = {
    "puma-windows-x86_64": {
        "url": "https://enhance-pet.s3.eu-central-1.amazonaws.com/awesome/beast-binaries-windows-x86_64.zip",
        "filename": "beast-binaries-windows-x86_64.zip",
        "directory": "beast-binaries-windows-x86_64",
    },
    "puma-linux-x86_64": {
        "url": "https://enhance-pet.s3.eu-central-1.amazonaws.com/awesome/beast-binaries-linux-x86_64.zip",
        "filename": "beast-binaries-linux-x86_64.zip",
        "directory": "beast-binaries-linux-x86_64",
    },
    "puma-mac-x86_64": {
        "url": "https://enhance-pet.s3.eu-central-1.amazonaws.com/awesome/beast-binaries-mac-x86_64.zip",
        "filename": "beast-binaries-mac-x86_64.zip",
        "directory": "beast-binaries-mac-x86_64",
    },
    "puma-mac-arm64": {
        "url": "https://enhance-pet.s3.eu-central-1.amazonaws.com/awesome/beast-binaries-mac-arm64.zip",
        "filename": "beast-binaries-mac-arm64.zip",
        "directory": "beast-binaries-mac-arm64",
    },
}


def check_device() -> str:
    """
    This function checks the available device for running predictions, considering CUDA and MPS (for Apple Silicon).

    Returns:
        str: The device to run predictions on, either "cpu", "cuda", or "mps".
    """
    # Check for CUDA
    if torch.cuda.is_available():
        device_count = torch.cuda.device_count()
        print(f" CUDA is available with {device_count} GPU(s). Predictions will be run on GPU.")
        return "cuda"
    # Check for MPS (Apple Silicon) Here for the future but not compatible right now
    elif torch.backends.mps.is_available():
        print(" Apple MPS backend is available. Predictions will be run on Apple Silicon GPU.")
        return "mps"
    elif not torch.backends.mps.is_built():
        print(" MPS not available because the current PyTorch install was not built with MPS enabled.")
        return "cpu"
    else:
        print(" CUDA/MPS not available. Predictions will be run on CPU.")
        return "cpu"
